<?php
/**
 * This file is part of the Katalysis WordPress plugin.
 *
 * (c) 2017 Katalysis <wpp@katalysis.io>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Katalysis;

require_once __DIR__ . '/utility.php';

class HTTPClient
{
    private $user_agent = '';

    public function __construct()
    {
        $blog_version = get_bloginfo('version');
        $blog_url = get_bloginfo('url');
        $this->user_agent = "WordPress/$blog_version; $blog_url";
    }

    /**
     * Send POST request to $url with $body.
     *
     * @param string $url
     * @param string $body
     * @return string
     */
    public function post($url, $body, $timeout = 5){
        //Utility::log("FEATHER-HTTP: POST> $url");
        //Utility::log("FEATHER-HTTP: POST> $body");
        return $this->request_and_get_resonse_body($url, array('method' => 'POST', 'body' => $body), $timeout);
    }

    public static function postonce($url, $body, $timeout = 5) {
        $http = new HTTPClient();
        return $http->post($url, $body, $timeout);
    }

    /**
     * Send GET request to $url with $body.
     *
     * @param string $url
     * @param string $body
     * @return string
     */
    public function get($url, $body, $timeout = 5)
    {
        //Utility::log("FEATHER-HTTP: GET> $url");
        //Utility::log("FEATHER-HTTP: GET> $body");
        return $this->request_and_get_resonse_body($url, array('method' => 'GET', 'body' => $body), $timeout);
    }

    public static function getonce($url, $body, $timeout = 5) {
        $http = new HTTPClient();
        return $http->get($url, $body, $timeout);
    }

    public function delete($url, $body, $timeout = 5){
        //Utility::log("FEATHER-HTTP: DELETE> $url");
        //Utility::log("FEATHER-HTTP: DELETE> $body");
        return $this->request_and_get_resonse_body($url, array('method' => 'DELETE', 'body' => $body), $timeout);
    }

    public static function deleteonce($url, $body, $timeout = 5) {
        $http = new HTTPClient();
        return $http->delete($url, $body, $timeout);
    }

    private function request_and_get_resonse_body($url, $args, $timeout)
    {
        $args['headers'] = array(
            'Accept' => 'application/json',
            'Content-Type' => 'application/json');
        $args['user-agent'] = $this->user_agent;
        $args['timeout'] = $timeout;

        $result = wp_remote_request($url, $args);
        if (is_wp_error($result))
        {
            $error = $result->get_error_message();
            //Utility::log("FEATHER-HTTP: < $url $error");

            throw new \Exception(
                "API request failed. Error: '$error'");
        }

        $body = wp_remote_retrieve_body($result);

        // Throw exception on error
        if ($result['response']['code'] != 200)
        {
            throw new \Exception(
                "API request failed ($url). Reply: '$body' (code {$result['response']['code']})");
        }

        return $body;
    }
}
