<?php
/**
 * This file is part of the Katalysis WordPress plugin.
 *
 * (c) 2017 Katalysis <wpp@katalysis.io>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Katalysis;

class Utility
{
    private function __construct() {}

    public static function log($data)
    {
        if (is_array($data) || is_object($data))
        {
            error_log(print_r($data, true));
            return;
        }

        error_log($data);
    }

    /**
     * dec2hex($number)
     *
     * @param string $number : A decimal number as a String.
     * @return string: Hex value of provided decimal string.
     */
    public static function dec_to_hex($number)
    {
        if ($number == '0')
        {
            return '0';
        }

        $hexvalues = array('0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F');
        $hexval = '';
        while ($number != '0')
        {
            $hexval = $hexvalues[bcmod($number, '16')].$hexval;
            $number = bcdiv($number, '16', 0);
        }
        return $hexval;
    }

    // Input: A hexadecimal number as a String.
    // Output: The equivalent decimal number as a String.
    public static function hex_to_dec($number)
    {
        $decvalues = array(
            '0' => '0', '1' => '1', '2' => '2', '3' => '3',
            '4' => '4', '5' => '5', '6' => '6', '7' => '7',
            '8' => '8', '9' => '9', 'A' => '10', 'B' => '11',
            'C' => '12', 'D' => '13', 'E' => '14', 'F' => '15');
        $decval = '0';
        $number = strrev($number);
        for($i = 0; $i < strlen($number); $i++)
        {
            $decval = bcadd(bcmul(bcpow('16',$i,0),$decvalues[$number{$i}]), $decval);
        }
        return $decval;
    }

    public static function str_to_hex($string)
    {
        $hex = '';
        for ($i=0; $i<strlen($string); $i++){
            $ord = ord($string[$i]);
            $hexCode = dechex($ord);
            $hex .= substr('0'.$hexCode, -2);
        }
        return strToUpper($hex);
    }

    public static function hex_to_str($hex)
    {
        $string='';
        for ($i=0; $i < strlen($hex)-1; $i+=2)
        {
            $string .= chr(hexdec($hex[$i].$hex[$i+1]));
        }
        return $string;
    }

    public static function split_content_head_tail($content, $numParagraphs)
    {
        $tagLength = strlen("</p>");
        if ($content == null || strlen($content)< $tagLength)
        {
            return array ($content, '');
        }

        $cutoff = 0;
        for ($i = 0; $i < (int)$numParagraphs; $i++)
        {
            $offset = strpos($content, "</p>", $cutoff + $tagLength);
            if ($offset !== false)
            {
                $cutoff = $offset;
            } else {
                // content has less paragraphs than needed
                return array ($content, '');
            }
        }

        $head = substr($content, 0, $cutoff + $tagLength);
        $tail = substr($content, $cutoff + $tagLength);

        return array($head, $tail);
    }

    public static function try_parse_float($dict, $verb, $default)
    {
        $text = isset($dict[$verb]) ? $dict[$verb]: strval($default);
        return floatval(str_replace(",", ".", $text));
    }

    public static function try_parse_int($dict, $verb, $default)
    {
        return isset($dict[$verb]) ? intval($dict[$verb]): $default;
    }

    public static function try_parse_bool($dict, $verb, $default)
    {
        return isset($dict[$verb]) ? boolval($dict[$verb]): $default;
    }
}