<?php
/**
 * This file is part of the Katalysis WordPress plugin.
 *
 * (c) 2018-2020 Katalysis <wpp@katalysis.io>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 *
 * @wordpress-plugin
 * Plugin Name:       Feather by Katalysis
 * Plugin URI:        https://feather.katalysis.io/
 * Description:       Enable micropayments (plus donations and subscriptions) for your blog.
 * Version:           2.8.8
 * Author:            Katalysis
 * Author URI:        https://katalysis.io
 * License:           GPL2
 * License URI:       https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain:       katalysis
 * Domain Path:       /languages
 */

require_once __DIR__ . '/katalysis-sdk/src/plugin_api.php';
require_once __DIR__ . '/katalysis-sdk/src/utility.php';

use Katalysis\PluginApi as KatalysisWrapper;

KatalysisWrapper::get_instance();

$rendering_priority = get_option("rendering_priority");

register_activation_hook( __FILE__,  function() {
    KatalysisWrapper::get_instance()->on_plugin_activated();
});
register_deactivation_hook( __FILE__,  function() {
    KatalysisWrapper::get_instance()->on_plugin_deactivated();
});
add_action('admin_init', function()
{
    KatalysisWrapper::get_instance()->on_wordpress_admin_init();
});
add_action('admin_menu', function()
{
    KatalysisWrapper::get_instance()->on_wordpress_admin_menu(null);
});
add_action('admin_enqueue_scripts', function($hook)
{
    KatalysisWrapper::get_instance()->on_wordpress_admin_enqueue_scripts($hook);
});
add_action('save_post', function ($post_id, $post, $update)
{
    if (wp_is_post_revision($post_id) || wp_is_post_autosave($post_id)) {
        return;
    }

    if ($update)
    {
        KatalysisWrapper::get_instance()->save_post_metadata($post, $_POST);
    }
}, 10, 3);
add_action('add_meta_boxes', function()
{
    $ignored_post_types = KatalysisWrapper::get_instance()->get_ignored_post_types();
    $post_types = array_diff(
        array_merge(array('post' => 'post'), get_post_types(array('_builtin' => FALSE))),
        $ignored_post_types
    );
    add_meta_box(
        'pwb_meta', 'Feather by Katalysis Options',
        function($post)
        {
            $view = KatalysisWrapper::get_instance()->get_post_settings_view($post);
            echo "$view";
        },
        $post_types, 'side', 'high');
});
add_filter('the_content', function($content)
{
    global $post;
    $rendering_priority = get_option("rendering_priority");

    return KatalysisWrapper::get_instance()->get_html_content($post, $content);
}, $rendering_priority);
add_action('wp_enqueue_scripts', function()
{
    KatalysisWrapper::get_instance()->on_wordpress_wp_enqueue_scripts();
});

function is_pricing_enabled_for_post($post_id)
{
    $post_settings = get_post_meta($post_id, '_katalysis_pricing', true);
    $legacy_enabled = isset($post_settings['enabled'])
        ? boolval($post_settings['enabled'])
        : false;
    return $legacy_enabled || KatalysisWrapper::get_instance()->get_enabled_state($post_id);
}
